/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xmlmap.impl;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;

import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;


public class InitializerImpl
{
	private ObjInfoHolder holder;
	private Object _validSupport;
	
	private List toInitialize = new ArrayList();
	private List initializing = new ArrayList();
	
	public InitializerImpl(ObjInfoHolder objInfoHolder, Object iValidSupport)
	{
		_validSupport = iValidSupport;
		holder = objInfoHolder;
		List objInfos = objInfoHolder.getAllObjInfo();
		for(int i=0; i<objInfos.size(); i++)
		{
			ObjInfo info = (ObjInfo)objInfos.get(i);
			if(info.getObject() == null)
				continue;
			if(!(info.getObject() instanceof IInitializable))
				continue;
			toInitialize.add(info);
		}
	}
	public void initializeAll()
	{
		while(!toInitialize.isEmpty())
		{
			ObjInfo info = (ObjInfo)toInitialize.get(0);
			// do not check state (must be TO_INTIALIZE)
			doInitialize(info);
		}
	}
	private boolean doCouldInitialize(Object object)
	{
		if(object == null)
			// --- object not instantiated by the XML instantiator
			return false;
		
		ObjInfo info = holder.getInfoFromObj(object);
		if(info == null)
			// this was not an instantiated object: exception?
			return true;
		
		if(object instanceof IInitializable)
		{
			// --- is it initialized, initiliazing or to initialize?
			if(initializing.contains(info))
			{
				// --- problem: cyclicity
				StringBuffer sb = new StringBuffer("Cyclic dependency found: ");
				for(int i=0; i<initializing.size(); i++)
				{
					ObjInfo inf = (ObjInfo)initializing.get(i);
					sb.append(inf.getObject().getClass().getName());
					sb.append(" --> ");
				}
				sb.append(info.getObject().getClass().getName());
				
				throw new RuntimeException(sb.toString());
			}
			else if(toInitialize.contains(info))
			{
				doInitialize(info);
			}
			else
			{
				// already initialized
			}
		}
		
		return !info.hasMessages(IInitializationSupport.ERROR);
	}
	private void doInitialize(ObjInfo info)
	{
//System.out.println("init: "+info.getNicePath());
		toInitialize.remove(info);
		initializing.add(info);
		try
		{
			((IInitializable)info.getObject()).initialize(new InitializationSupportImpl(info), _validSupport);
		}
		catch(Throwable t)
		{
			StringWriter w = new StringWriter();
			t.printStackTrace(new PrintWriter(w));
			info.addMessage(null, InitializationSupportImpl.ERROR, w.toString());
		}
		initializing.remove(info);
	}
	// =======================================================================
	// === IInitializationSupport implementation
	// =======================================================================
	private class InitializationSupportImpl implements IInitializationSupport
	{
		private ObjInfo info;
		
		public InitializationSupportImpl(ObjInfo info)
		{
			this.info = info;
		}
		public void addValidationMessage(Object fromNode, String mappingName, int severity, String message)
		{
			if(fromNode == info.getObject())
			{
				info.addMessage(mappingName, severity, message);
			}
			else
			{
				ObjInfo info2 = holder.getInfoFromObj(fromNode);
				if(info2 != null)
					info2.addMessage(mappingName, severity, message);
			}
		}
		public boolean initialize(Object object)
		{
			// TODO: check whether the object is an ancestor (forbidden!)
			boolean dependencyCouldInit = doCouldInitialize(object);
//			if(!dependencyCouldInit)
//				info.addMessage(null, IInitializationSupport.ERROR, "Dependency failed to initialize: "+object);
			return dependencyCouldInit;
		}
	}
}
