/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xmlmap.impl;

import java.io.PrintWriter;
import java.net.URL;


public class ModelMessageImpl implements IXmlModelMessage
{
	private ObjInfo _oi;
	private String _mapping;
	private int _type;
	private String _msg;
	
	public ModelMessageImpl(ObjInfo iObjInfo, String iMappingName, int iType, String iMsg)
	{
		_oi = iObjInfo;
		_mapping  = iMappingName;
		_type = iType;
		_msg = iMsg;
	}
	public Object getObject()
	{
	    return _oi.getObject();
	}
	public ObjInfo getInfo()
	{
		return _oi;
	}
	public String getMappingName()
	{
	    return _mapping;
	}
	public String getMessage()
	{
	    return _msg;
	}
	public int getSeverity()
	{
	    return _type;
	}
	public URL getFile()
	{
		return _oi.getXmlFile();
	}
	public int getLine()
	{
//		return _oi.getLineNumber();
		return _oi.getMappingLocation(_mapping);
	}
	public void dump(PrintWriter iPrinter)
	{
		switch(_type)
		{
		case IXmlModelMessage.INST_ERROR:
			iPrinter.println("+ Instanciation Error: "+_msg);
			break;
		case IXmlModelMessage.INST_WARNING:
			iPrinter.println("+ Instanciation Warning: "+_msg);
			break;
		case IXmlModelMessage.INST_INFO:
			iPrinter.println("+ Instanciation Info: "+_msg);
			break;
		case IXmlModelMessage.VALID_ERROR:
			iPrinter.println("+ Validation Error: "+_msg);
			break;
		case IXmlModelMessage.VALID_WARNING:
			iPrinter.println("+ Validation Warning: "+_msg);
			break;
		case IXmlModelMessage.VALID_INFO:
			iPrinter.println("+ Validation Info: "+_msg);
			break;
		}
	}
}