/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xmlmap.impl;

import java.io.PrintWriter;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.ow2.opensuit.xmlmap.mapping.ElementMapping;
import org.ow2.opensuit.xmlmap.mapping.FieldMapping;
import org.ow2.opensuit.xmlmap.schema.ISchemaElement;


public class ObjInfo
{
	private ObjInfo parent;
	private FieldMapping parentMapping;
	private int childIndex = -1;
	private Object obj;
	private ISchemaElement schemaElt;
	private List messages;
	private URL xmlFile;
	private int lineNumber = -1;
//	private int colNumber = -1;
	private HashMap name2Loc;
	
	public ObjInfo(ObjInfo iParent, FieldMapping iParentMapping, ISchemaElement iSchemaElt, Object iObj)
	{
		parent = iParent;
		parentMapping = iParentMapping;
		schemaElt = iSchemaElt;
		obj = iObj;
	}
	public ObjInfo(ObjInfo iParent, FieldMapping iParentMapping, int iChildIdx, ISchemaElement iSchemaElt, Object iObj)
	{
		parent = iParent;
		parentMapping = iParentMapping;
		schemaElt = iSchemaElt;
		childIndex = iChildIdx;
		obj = iObj;
	}
	public void setMappingLocation(String name, int line)
	{
		if(name2Loc == null)
			name2Loc = new HashMap(1);
		name2Loc.put(name, new Integer(line));
	}
	public int getMappingLocation(String name)
	{
		if(name == null || name2Loc == null)
			return lineNumber;
		Integer mappingLine = (Integer)name2Loc.get(name);
		return mappingLine == null ? lineNumber : mappingLine.intValue();
	}
	public void setLocation(URL xmlFile, int lineNumber)
    {
    	this.xmlFile = xmlFile;
		this.lineNumber = lineNumber;
    }
	public URL getXmlFile()
    {
    	return xmlFile;
    }
	public int getLineNumber()
	{
		return lineNumber;
	}
	/*
	public int getColNumber()
	{
		return colNumber;
	}
	public void setColNumber(int colNumber)
	{
		this.colNumber = colNumber;
	}
	*/
	public Object getObject()
	{
		return obj;
	}
	public ISchemaElement getSchemaElement()
	{
		return schemaElt;
	}
	public ObjInfo getParent()
	{
		return parent;
	}
	public FieldMapping getParentMapping()
	{
		return parentMapping;
	}
	public int getParentChildIndex()
	{
		return childIndex;
	}
	/*
	public void addValidationMessage(Object iFromNode, String iMappingName, int iSeverity, String iMessage)
	{
		addMessage(iMappingName, iSeverity, iMessage);
	}
	*/
	public void clearMessages()
	{
		if(messages == null)
			return;
		messages.clear();
	}
	public void addMessage(String iMappingName, int iType, String iMessage)
	{
		if(messages == null)
			messages = new ArrayList(1);
		messages.add(new ModelMessageImpl(this, iMappingName, iType, iMessage));
	}
	public int countMessages(int treshold)
	{
		if(messages == null)
			return 0;
		int count = 0;
		for(int i=0; i<messages.size(); i++)
			if(((ModelMessageImpl)messages.get(i)).getSeverity() <= treshold)
				count++;
		return count;
	}
	public List getMessages(int treshold)
	{
		if(messages == null)
			return null;
		List ret = new ArrayList();
		for(int i=0; i<messages.size(); i++)
			if(((ModelMessageImpl)messages.get(i)).getSeverity() <= treshold)
				ret.add(messages.get(i));
		return ret;
	}
	public boolean hasMessages(int treshold)
	{
		if(messages == null)
			return false;
		for(int i=0; i<messages.size(); i++)
			if(((ModelMessageImpl)messages.get(i)).getSeverity() <= treshold)
				return true;
		return false;
	}
	private String getNiceName()
	{
		if(obj == null)
			return "Doc";
		
		StringBuffer sb = new StringBuffer();
		sb.append(schemaElt.getSchema().getName());
		sb.append(":");
		sb.append(schemaElt.getTagName());
		
		ElementMapping mappings = (ElementMapping)schemaElt.getSchema().getMethod().getElementMappings(obj.getClass(), true);
		FieldMapping[] v = mappings.getMappings(FieldMapping.ATTRIBUTE);
		for(int i=0; i<v.length; i++)
		{
			FieldMapping field = v[i];
			if(!field.isUseRequired())
				continue;
            try
            {
				field.getField().setAccessible(true);
            	Object value = field.getField().get(obj);
				if(value != null)
				{
					sb.append(" ");
					sb.append(field.getName());
					sb.append("=");
					sb.append("'");
					if(schemaElt.getSchema().getMethod().isEnumeration(field.getBaseClass()))
					{
						sb.append(schemaElt.getSchema().getMethod().getEnumItemName(value));
					}
					else
					{
						sb.append(String.valueOf(value));
					}
					sb.append("'");
				}
            }
            catch(IllegalArgumentException e)
            {
	            // TODO Auto-generated catch block
	            e.printStackTrace();
            }
            catch(IllegalAccessException e)
            {
	            // TODO Auto-generated catch block
	            e.printStackTrace();
            }
		}
		return sb.toString();
	}
	private String getNicePath()
	{
		if(xmlFile != null && lineNumber >= 0)
//			return xmlFile+"(l "+lineNumber+" c "+colNumber+") > "+getNiceName();
			return xmlFile+"(l "+lineNumber+") > "+getNiceName();
		
		if(parent == null)
		{
			if(xmlFile != null)
				return xmlFile.toExternalForm()+" > "+getNiceName();
			return getNiceName();
		}
		if(xmlFile != null && !xmlFile.equals(parent.xmlFile))
		{
			// --- this is the root component for this file
			return xmlFile.toExternalForm()+" > "+getNiceName();
		}
		
		return parent.getNicePath()+" > "+getNiceName();
	}
	/*
	private String getId()
	{
		if(parent == null)
			return "root";
		if(parentMapping.getType() == FieldMapping.DIRECT_CHILD_ELT)
		{
			if(childIndex < 0)
				return parentMapping.getName();
			return parentMapping.getName()+"."+childIndex;
		}
		else
		{
			if(childIndex < 0)
				return "_";
			return String.valueOf(childIndex);
		}
	}
	private String getIdPath()
	{
		if(parent == null)
			return "/"+getId();
		if(parentMapping.getType() == FieldMapping.DIRECT_CHILD_ELT)
			return parent.getIdPath()+"/"+getId();
		
		return parent.getIdPath()+"/"+parentMapping.getName()+"/"+getId();
	}
	*/
	public void dumpMessages(int treshold, PrintWriter iPrinter)
	{
		List errors = getMessages(treshold);
		if(errors == null || errors.size() == 0)
			return;
		
		iPrinter.println("["+getNicePath()+"]:");
		for(int i=0; i<errors.size(); i++)
				((ModelMessageImpl)errors.get(i)).dump(iPrinter);
		iPrinter.println(" ");
	}
}