/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xmlmap.mapping;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.List;


public class ElementMapping
{
	private static FieldMapping[] MAPPED_ARRAY_TYPE = {};

	private Constructor _defaultCS;
	private List _mappings;
	private Class _class;
	private MappingError[] _mappingErrors;
	
	protected ElementMapping(Class mappedClass, Constructor iCS, List mappings, MappingError[] mappingErrors)
	{
		_class = mappedClass;
		_mappingErrors = mappingErrors;
		_defaultCS = iCS;
		_mappings = mappings;
	}
	public Class getMappedClass()
	{
		return _class;
	}
	public MappingError[] getMappingErrors()
	{
		return _mappingErrors;
	}
	public Constructor getDefaultConstructor()
	{
		return _defaultCS;
	}
	/**
	 * Returns the list of field mappings that match the give type(s) combination
	 * type 0 returns all mappings.
	 * @param iTypes
	 * @return
	 */
	public FieldMapping[] getMappings(int iTypes)
	{
		if(iTypes == 0)
			return (FieldMapping[])_mappings.toArray(MAPPED_ARRAY_TYPE);
		
		ArrayList ret = new ArrayList();
		for(int i=0; i<_mappings.size(); i++)
		{
			FieldMapping mapping = (FieldMapping)_mappings.get(i);
			if((mapping.getType() & iTypes) != 0)
				ret.add(mapping);
		}
		return (FieldMapping[])ret.toArray(MAPPED_ARRAY_TYPE);
	}
	public FieldMapping getMappingByNameAndType(String iName, int iType)
	{
		for(int i=0; i<_mappings.size(); i++)
		{
			FieldMapping mapping = (FieldMapping)_mappings.get(i);
			if(mapping.getType() == iType && mapping.getName().equals(iName))
				return mapping;
		}
		return null;
	}
	public FieldMapping getCollectedMapping(Class iClass)
	{
		for(int i=0; i<_mappings.size(); i++)
		{
			FieldMapping mapping = (FieldMapping)_mappings.get(i);
			if(mapping.getType() == FieldMapping.DIRECT_CHILD_ELT && mapping.getBaseClass().isAssignableFrom(iClass))
				return mapping;
		}
		return null;
	}
}
