/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xmlmap.mapping;

import java.lang.reflect.Field;

public class FieldMapping
{
	/**
	 * This is the type of a Field that is mapped to an XML attribute.
	 */
	public static final int ATTRIBUTE = 1;
	/**
	 * This is the type of a Field that is mapped to one or several elements in a named child element.
	 */
	public static final int CHILD_ELT = 2;
	/**
	 * This is the type of a Field that is mapped to one or several XML direct child elements.
	 */
	public static final int DIRECT_CHILD_ELT = 4;
	/**
	 * This is the type of a Field that is mapped to the content of the element.
	 */
	public static final int CONTENT = 8;
	/**
	 * This is the type of a Field that is mapped to the parent object in instantiated the model.
	 */
	public static final int PARENT = 16;
	/**
	 * This is the type of a Field that is mapped to the root object in the instantiated model.
	 */
	public static final int ROOT = 32;
	/**
	 * This is the type of a Field that is mapped to the closest ancestor that matches the class.
	 */
	public static final int ANCESTOR = 64;
	
	private Field field;
	private int type;
	private String name;
	private int minOccurs = 1;
	private int maxOccurs = 1;
	
	public FieldMapping(Field field)
	{
		this.field = field;
	}
	
	public String toString()
	{
		if(type == ATTRIBUTE)
		{
			return field.getName()+": "+(minOccurs == 0 ? "optional" : "required")+" attribute '"+name+"' of type "+field.getType().getName();
		}
		else if(type == CONTENT)
		{
			return field.getName()+": HTML content";
		}
		else if(type == PARENT)
		{
			return field.getName()+": parent node";
		}
		else if(type == PARENT)
		{
			return field.getName()+": root node";
		}
		if(field.getType().isArray() && field.getType() != byte[].class)
		{
			return field.getName()+": "+(type == CHILD_ELT ? "indirect children '"+name+"'" : "direct children")+" ["+minOccurs+","+((maxOccurs == Integer.MAX_VALUE ? "unbounded" : String.valueOf(maxOccurs)))+"] of type "+field.getType().getComponentType().getName();
		}
		return field.getName()+": "+(minOccurs == 0 ? "optional" : "required")+" "+(type == CHILD_ELT ? "indirect child '"+name+"'" : "direct child")+" of type "+field.getType().getName();
	}
	public int getType()
	{
		return type;
	}
	public String getName()
	{
		return name;
	}
	public Field getField()
	{
		return field;
	}
	public Class getBaseClass()
	{
		return field.getType().isArray() && field.getType() != byte[].class ? field.getType().getComponentType() : field.getType();
	}
	public boolean isUseRequired()
	{
		return minOccurs > 0;
	}
	public int getMinOccurs()
	{
		return minOccurs;
	}
	public int getMaxOccurs()
	{
		return maxOccurs;
	}

	public void setType(int type)
	{
		this.type = type;
	}

	public void setName(String name)
	{
		this.name = name;
	}

	public void setMinOccurs(int minOccurs)
	{
		this.minOccurs = minOccurs;
	}

	public void setMaxOccurs(int maxOccurs)
	{
		this.maxOccurs = maxOccurs;
	}
	
}
