/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xmlmap.schema;

import org.ow2.opensuit.xmlmap.mapping.IMappingMethod;
import org.w3c.dom.Document;


/**
 * Represents relationships between all classes that conform to 
 * implicit Xml Mapping rule in a given root package.<br>
 * A bunch of objects that take advantage of implicit Xml Mappings should
 * be organized within a schema.<br>
 * Basically this is just sharing a root java package.
 */
public interface ISchema
{
	/**
	 * Returns the schema name (that is also the prefix).
	 */
	String getName();
	/**
	 * Returns the schema location (this is the xsd file).
	 */
	String getLocator();
	/**
	 * Returns the schema root package (that is also the schema namespace)
	 */
	String getRootPackage();
	/**
	 * Returns a list of schemas referenced by this schema.
	 * @return
	 */
	ISchema[] getUsedSchemas();
	/**
	 * Returns all elements belonging to this schema.
	 * @return
	 */
	ISchemaElement[] getAllElements();
	/**
	 * Builds the schema (i.e. looks for all elements) by searching the classpath for classes.
	 * @throws Exception
	 */
	void buildFromClassPath() throws Exception;
	
	/**
	 * Builds the schema (i.e. looks for all elements) by searching the given source directory.
	 * @throws Exception
	 */
//	void buildFromSource(String iSourceDir) throws Exception;
	
	/**
	 * Builds the schema (i.e. looks for all elements) by parsing the XSD file.
	 * @throws Exception
	 */
//	void buildFromXsd(Document iXsdDoc) throws Exception;
	
	/**
	 * Builds the schema (i.e. looks for all elements) by parsing the XSD file.
	 * @throws Exception
	 */
//	void buildFromXsd(InputStream iInput) throws Exception;
	
	/**
	 * Determines whether this schema has been loaded.
	 * @return
	 */
	boolean isFullyBuilt();
	
	/**
	 * Generates the XSD structure for this schema.
	 * @param iXmlDoc
	 */
	void generateXSD(Document iXmlDoc);
	
//	void writeXSD(PrintWriter iWriter);
	
	ISchemaElement getElement(Class iClass);
	
	IMappingMethod getMethod();
}
