/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xmlmap.utils;

import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

public class XmlHelper
{
	/**
	 * Serializes the given Xml structure.
	 * Output is indented, encoded in utf-8.
	 * @param iNode
	 * @param iOutput
	 * @throws UnsupportedEncodingException
	 * @throws TransformerException
	 */
	public static void serializeXml(Node iNode, OutputStream iOutput) throws UnsupportedEncodingException, TransformerException
	{
		String encoding = "utf-8";
		TransformerFactory transFactory = TransformerFactory.newInstance();
		try {
			transFactory.setAttribute("indent-number", new Integer(2));
		} catch(IllegalArgumentException e) {}
		Transformer transformer = transFactory.newTransformer();
		try {
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");
		} catch(IllegalArgumentException e) {}
		try {
			transformer.setOutputProperty(OutputKeys.ENCODING, encoding);
		} catch(IllegalArgumentException e) {}
// no		transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "2");
		DOMSource source = new DOMSource(iNode);
		StreamResult result = new StreamResult(new OutputStreamWriter(iOutput, encoding));
		transformer.transform(source, result);
	}
	public static String getContent(Element iNode)
	{
//		String html = ((Text)iNode.getFirstChild()).getData();
		/*
		String html = iNode.toString();
		int idx1 = html.indexOf('>');
		int idx2 = html.lastIndexOf('<');
		if(idx1 > 0 && idx2 > 0)
			html = html.substring(idx1+1, idx2);
		*/
		StringBuffer sb = new StringBuffer();
		getContent(iNode, sb);
		return sb.toString();
	}
	private static void getContent(Element iNode, StringBuffer sb)
	{
		NodeList nodes = iNode.getChildNodes();
		for (int i=0; i<nodes.getLength(); i++)
		{
			Node n = nodes.item(i);
			if(n instanceof Text) // CData extends Text
			{
				Text t = (Text)n;
				sb.append(t.getData());
			}
			else if (n instanceof Element)
			{
				Element e = (Element)n;
				sb.append("<");
				sb.append(e.getTagName());
				NamedNodeMap attrs = e.getAttributes();
				for (int j=0; j<attrs.getLength(); j++)
				{
					if(attrs.item(j) instanceof Attr)
					{
						Attr a = (Attr)attrs.item(j);
						sb.append(' ');
						sb.append(a.getNodeName());
						if(a.getNodeValue() != null)
						{
							sb.append("=\"");
							sb.append(a.getNodeValue());
							sb.append("\"");
						}
					}
				}
				if(!e.hasChildNodes())
				{
					sb.append("/>");
				}
				else
				{
					sb.append(">");
					getContent(e, sb);
					sb.append("</");
					sb.append(e.getTagName());
					sb.append(">");
				}
			}
		}
	}
	/*
	public static Element[] getChildElements(Element iElt)
	{
		NodeList list = iElt.getChildNodes();
		Vector elts = new Vector();
		for(int i=0; i<list.getLength(); i++)
		{
			if(list.item(i) instanceof Element)
				elts.addElement((Element)list.item(i));
		}
		Element[] ret = new Element[elts.size()];
		elts.copyInto(ret);
		return ret;
	}
	public static Element[] getChildElementsByTag(Element iNode, String iNodeName)
	{
		NodeList list = iNode.getChildNodes();
		Vector elts = new Vector();
		for(int i=0; i<list.getLength(); i++)
		{
			if(list.item(i) instanceof Element)
			{
				if(((Element)list.item(i)).getTagName().equals(iNodeName))
					elts.addElement((Element)list.item(i));
			}
		}
		Element[] ret = new Element[elts.size()];
		elts.copyInto(ret);
		return ret;
	}
	*/
}
