/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xmlmap.utils;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

public class XsdHelper
{
	private static DateFormat XML_DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
	

	public static String getSimpleType(Class target)
	{
		if(target == int.class || target == Integer.class)
		{
			return "int";
		}
		else if(target == short.class || target == Short.class)
		{
			return "short";
		}
		else if(target == long.class || target == Long.class)
		{
			return "long";
		}
		else if(target == BigInteger.class)
		{
			return "integer";
		}
		else if(target == float.class || target == Float.class || target == double.class || target == Double.class || target == BigDecimal.class)
		{
			return "decimal";
		}
		else if(target == boolean.class || target == Boolean.class)
		{
			return "boolean";
		}
		else if(target == Date.class || target == Calendar.class)
		{
			// TODO: other types
			return "dateTime";
		}
		else if(target == byte[].class)
		{
			return "hexBinary";
		}
		else if(target == String.class)
		{
			return "string";
		}
		
		// ???
		return null;
	}
	
	public static Object parseSimpleType(String iValue, Class target) throws Exception
	{
		if(iValue == null)
			// TODO: return default value for primitive types?
			return null;
		
		if(target == int.class || target == Integer.class)
		{
			BigInteger bi = new BigInteger(iValue);
			if(bi.compareTo(BigInteger.valueOf(Integer.MIN_VALUE)) < 0 || bi.compareTo(BigInteger.valueOf(Integer.MAX_VALUE)) > 0)
				throw new NumberOutOfBoundsException(Integer.MIN_VALUE, Integer.MAX_VALUE);
			return new Integer(bi.intValue());
		}
		else if(target == short.class || target == Short.class)
		{
			BigInteger bi = new BigInteger(iValue);
			if(bi.compareTo(BigInteger.valueOf(Short.MIN_VALUE)) < 0 || bi.compareTo(BigInteger.valueOf(Short.MAX_VALUE)) > 0)
				throw new NumberOutOfBoundsException(Short.MIN_VALUE, Short.MAX_VALUE);
			return new Short(bi.shortValue());
		}
		else if(target == long.class || target == Long.class)
		{
			BigInteger bi = new BigInteger(iValue);
			if(bi.compareTo(BigInteger.valueOf(Long.MIN_VALUE)) < 0 || bi.compareTo(BigInteger.valueOf(Long.MAX_VALUE)) > 0)
				throw new NumberOutOfBoundsException(Long.MIN_VALUE, Long.MAX_VALUE);
			return new Long(bi.longValue());
		}
		else if(target == BigInteger.class)
		{
			return new BigInteger(iValue);
		}
		else if(target == float.class || target == Float.class)
		{
			/*
			BigDecimal bd = new BigDecimal(iValue);
//			if(bd.compareTo(BigDecimal.valueOf(Float.MIN_VALUE)) < 0 || bd.compareTo(BigDecimal.valueOf(Float.MAX_VALUE)) > 0)
//				throw new NumberOutOfBoundsException(Long.MIN_VALUE, Long.MAX_VALUE);
			return new Float(bd.floatValue());
			*/
			return new Float(iValue);
		}
		else if(target == double.class || target == Double.class)
		{
			return new Double(iValue);
		}
		else if(target == BigDecimal.class)
		{
			return new BigDecimal(iValue);
		}
		else if(target == boolean.class || target == Boolean.class)
		{
			if("true".equalsIgnoreCase(iValue))
				return Boolean.TRUE;
			else
				return Boolean.FALSE;
		}
		else if(target == Date.class)
		{
			return XML_DATE_FORMAT.parse(iValue);
		}
		else if(target == Calendar.class)
		{
			Calendar cal = Calendar.getInstance();
			cal.setTime(XML_DATE_FORMAT.parse(iValue));
			return cal;
		}
		else if(target == String.class)
		{
			return iValue;
		}
		else if(target == byte[].class)
		{
			return parseHex(iValue);
		}

		// ???
		return null;
	}
	public static String formatSimpleType(Object iValue, Class target)
	{
		if(iValue == null)
			// TODO: return default value for primitive types?
			return null;
		
		if(target == int.class || target == Integer.class)
		{
			return iValue.toString();
		}
		else if(target == short.class || target == Short.class)
		{
			return iValue.toString();
		}
		else if(target == long.class || target == Long.class)
		{
			return iValue.toString();
		}
		else if(target == BigInteger.class)
		{
			return iValue.toString();
		}
		else if(target == float.class || target == Float.class)
		{
			return iValue.toString();
		}
		else if(target == double.class || target == Double.class)
		{
			return iValue.toString();
		}
		else if(target == BigDecimal.class)
		{
			return iValue.toString();
		}
		else if(target == boolean.class || target == Boolean.class)
		{
			return iValue.toString();
		}
		else if(target == Date.class)
		{
			return XML_DATE_FORMAT.format(iValue);
		}
		else if(target == Calendar.class)
		{
			return XML_DATE_FORMAT.format(((Calendar)iValue).getTime());
		}
		else if(target == String.class)
		{
			return (String)iValue;
		}
		else if(target == byte[].class)
		{
			return formatHex((byte[])iValue);
		}

		// ???
		return null;
	}
	// ============================================================================
	// === Hexadecimal Helpers
	// ============================================================================
	private static final byte[] parseHex(String iString) throws Exception
	{
		if( iString==null ) return null;
		if( (iString.length() % 2) != 0 )
			throw new Exception("Not an Hexadecimal string: "+iString);
		
		char [] string= iString.toCharArray();
		byte [] result= new byte[iString.length()/2];
		for( int i= 0; i<result.length; i++ )
		{
			int high = getHexValue(string[i*2]);
			int low = getHexValue(string[i*2+1]);
			result[i] = (byte)((high << 4) + low);
		}
		return result;
	}
	private static int getHexValue(char iChar) throws Exception
	{
		if(iChar >= '0' && iChar <= '9')
			return iChar - '0';
		if(iChar >= 'a' && iChar <= 'f')
			return iChar - 'a' + 10;
		if(iChar >= 'A' && iChar <= 'F')
			return iChar - 'A' + 10;
		throw new Exception("Not an hexadecimal character: '"+iChar+"'.");
	}
	private static final char [] _encodeTable= {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
	/**
	 * Ecrit des donnes binaires (octets) en base 64
	 */
	private static final String formatHex( byte[] iData )
	{
		StringBuffer result= new StringBuffer();
		for (int i = 0; i < iData.length; i++)
		{
			int b= iData[i];
			if(b < 0)
				b+= 256;
			result.append(_encodeTable[b>>4]);
			result.append(_encodeTable[b&0xF]);
		}
		return result.toString();
	}

	// ============================================================================
	// === NumberOutOfBoundsException
	// ============================================================================
	public static class NumberOutOfBoundsException extends IllegalArgumentException
	{
		private long min;
		private long max;
		
		public NumberOutOfBoundsException(long min, long max)
		{
			super();
			this.min = min;
			this.max = max;
		}
		public long getMin()
		{
			return min;
		}
		public long getMax()
		{
			return max;
		}
	}
	// ============================================================================
	// === ArrayReadOnlyCollection
	// ============================================================================
	/*
	private static class ArrayReadOnlyCollection implements Collection<Object>
	{
		private int len;
		private Object array;
		
		private ArrayReadOnlyCollection(Object array)
		{
			this.array = array;
			len = Array.getLength(array);
		}
		public boolean add(Object o)
		{
			throw new UnsupportedOperationException("This collection may be used for read-only access.");
		}
		public boolean addAll(Collection<? extends Object> c)
		{
			throw new UnsupportedOperationException("This collection may be used for read-only access.");
		}
		public void clear()
		{
			throw new UnsupportedOperationException("This collection may be used for read-only access.");
		}
		public boolean contains(Object obj)
		{
			if(obj == null)
			{
				for(Object o : this)
				{
					if(o == null)
						return true;
				}
			}
			else
			{
				for(Object o : this)
				{
					if(obj.equals(o))
						return true;
				}
			}
			return false;
		}
		public boolean containsAll(Collection<?> c)
		{
			for(Object o : c)
			{
				if(!this.contains(o))
					return false;
			}
			return true;
		}
		@Override
		public boolean equals(Object obj)
		{
			if(!(obj instanceof ArrayReadOnlyCollection))
				return false;
			ArrayReadOnlyCollection c = (ArrayReadOnlyCollection)obj;
				
			return array.equals(c.array);
		}
		@Override
		public int hashCode()
		{
			return array.hashCode();
		}
		public boolean isEmpty()
		{
			return len == 0;
		}
		public Iterator<Object> iterator()
		{
			return new ArrayIterator();
		}
		public boolean remove(Object o)
		{
			throw new UnsupportedOperationException("This collection may be used for read-only access.");
		}
		public boolean removeAll(Collection<?> c)
		{
			throw new UnsupportedOperationException("This collection may be used for read-only access.");
		}
		public boolean retainAll(Collection<?> c)
		{
			throw new UnsupportedOperationException("This collection may be used for read-only access.");
		}
		public int size()
		{
			return len;
		}
		public Object[] toArray()
		{
			if(!array.getClass().getComponentType().isPrimitive())
				return (Object[])array;
			// --- we have to turn the primitive array into its Class typed equivalent
			Object ret = Array.newInstance(getNonPrimitiveClass(array.getClass().getComponentType()), len);
//			System.arraycopy(array, 0, ret, 0, len);
			for(int i=0; i<len; i++)
				Array.set(ret, i, Array.get(array, i));
			return (Object[])ret;
		}
		public <T extends Object> T[] toArray(T[] a)
		{
			if(a.length < len)
			{
				if(a.getClass().isAssignableFrom(array.getClass()))
					// --- array is of good type
					return (T[])array;
				// --- a is too narrow or array is not of good type
				a = (T[])Array.newInstance(a.getClass(), len);
			}
			for(int i=0; i<len; i++)
				a[i] = (T)Array.get(array, i);
			return a;
		}
		
		private class ArrayIterator implements Iterator<Object>
		{
			private int index = 0;
			public boolean hasNext()
			{
				return index < len;
			}
			public Object next()
			{
				return Array.get(array, index++);
			}
			public void remove()
			{
				throw new UnsupportedOperationException("This iterator may be used for read-only access.");
			}
		}
	}
	*/

}
