/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xmlmap.utils.minidom;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Set;
import java.util.Vector;


/**
 * Light and simplistic XML parser.
 */
public class XmlElement
{
	private String namespaceURI;
	private String localName;
	private String tagName;
	private HashMap attributes;
	private StringBuffer content;
	private List children;
	private int line = -1;
	private int col = -1;

	/**
	 * Constructor for a new XMLNode.
	 */
	public XmlElement( String uri, String localName, String name )
	{
		this.namespaceURI = uri;
		this.localName = localName;
		this.tagName = name;
	}
	public void setLocation(int line, int col)
	{
		this.line = line;
		this.col = col;
	}
	public int getLine()
	{
		return line;
	}
	public int getCol()
	{
		return col;
	}
	/**
	 * Insert a new Node at end of the children list.
	 */
	public void addChild( XmlElement iNode )
	{
		if( iNode==null ) return;
		if(children == null) children= new Vector(1);
		children.add( iNode );
	}
	/**
	 * Remove a Node from the children list.
	 */
	public void removeChild( XmlElement iNode )
	{
		if(children == null) return;
		children.remove( iNode );
	}
	public void addAttribute( XmlAttribute iAttr )
	{
		if( iAttr==null ) return;
		if(attributes == null) attributes= new HashMap(1);
		attributes.put( iAttr.getName(), iAttr );
	}

	/**
	 * Get an attribute value.
	 * iDefault is returned if attribute not set.
	 */
	public String getAttribute( String iName )
	{
		if(attributes == null)
			return null;
		XmlAttribute ret= (XmlAttribute)(attributes.get( iName ));
		if( ret==null )
			return null;
		return ret.getValue();
	}
	/**
	 * Get list of attributes
	 */
	public Set getAttributeNames()
	{
		if(attributes == null)
			return null;
		return attributes.keySet();
	}
	public Collection getAttributes()
	{
		if(attributes == null)
			return null;
		return attributes.values();
	}

	/**
	 * Get list of child nodes.
	 */
	public List getChildren()
	{
		if(children == null)
			// do not return null
			return Collections.EMPTY_LIST;
		return children;
	}
	public XmlElement getFirstChild()
	{
		if(children == null || children.size() == 0)
			return null;
		return (XmlElement)children.get(0);
	}
	/**
	 * Get text content
	 */
	public String getContent()
	{
		return content == null ? null : content.toString();
	}
	public void appendContent(char[] chars, int start, int len)
	{
		if(content == null)
			content = new StringBuffer(len);
		content.append(chars, start, len);
	}
	/**
	 * set the node content before its child nodes.
	 */
	public void setContent( String iContent )
	{
		if(iContent == null)
			content = null;
		else
			content = new StringBuffer(iContent);
	}
	/**
	 * Returns the element namespace.
	 * @return
	 */
	public String getNamespaceURI()
	{
		return namespaceURI;
	}
	/**
	 * Returns the local tag name (without any prefix)
	 * @return
	 */
	public String getLocalName()
	{
		return localName;
	}
	/**
	 * Returns the complete tag name (including the possible prefix)
	 * @return
	 */
	public String getTagName()
	{
		return tagName;
	}
	
}
